<html>
  <head>
    <title>Register an account</title>
  </head>
  <body>
    <h1>Register an account</h1>

    {% if message %}
      <p>{{ message }}</p>
    {% endif %}

    <h2>Enter your details</h2>
    <form action="{{ action }}" method="post">
      {{ form }}
      <p>
        Since you are using a Twitter account, you don't need to set a
        password. You can set one later if you want.
      </p>
      <p><input type="submit" value="Register"></p>
    </form>
    <p>
      Your <strong>{{ screen_name }}</strong> Twitter account will
      be associated with this account.
      (<a href="../logout/?next={{ no_thanks }}">no thanks</a>)
    </p>
  </body>
</html>
